@@ -76,6 +76,7 @@ gPodderSettings = {
     'episode_list_descriptions': (bool, True),
     'show_toolbar': (bool, True),
     'ipod_write_gtkpod_extended': (bool, False),
+    'mp3_player_delete_played': (bool, False),
     
     # Tray icon and notification settings
     'display_tray_icon': (bool, False),
@@ -106,11 +107,18 @@ gPodderSettings = {
     'create_m3u_playlists': (bool, False),
     'max_episodes_per_feed': (int, 200),
     'mp3_player_use_scrobbler_log': (bool, False),
+    'mp3_player_max_filename_length': (int, 100),
     'show_podcast_url_entry': (bool, True),
     'maemo_allow_custom_player': (bool, False),
-
+    'rockbox_copy_coverart' : (bool, False),
+    'rockbox_coverart_size' : (int, 100),
+    'experimental_file_naming': (bool, False),
+    'podcast_list_icon_size': (int, 32),
+    'cmd_all_downloads_complete': (str, ''),
+    'cmd_download_complete': (str, ''),
+    
     # Hide the cover/pill from the podcast sidebar when it gets too small
-    'podcast_sidebar_save_space': (bool, True),
+    'podcast_sidebar_save_space': (bool, False),
 
     # Window and paned positions
     'main_window_x': ( int, 100 ),
@@ -133,8 +141,8 @@ class Config(dict):
         self.__section = 'gpodder-conf-1'
         self.__ignore_window_events = False
         self.__observers = []
-        # Name, Type, Value, Type(python type), Editable?, Font weight
-        self.__model = gtk.ListStore(str, str, str, object, bool, int)
+        # Name, Type, Value, Type(python type), Editable?, Font style, Boolean?, Boolean value
+        self.__model = gtk.ListStore(str, str, str, object, bool, int, bool, bool)
 
         atexit.register( self.__atexit)
 
@@ -300,10 +308,11 @@ class Config(dict):
 
             self[key] = value
             if value == default:
-                weight = pango.WEIGHT_NORMAL
+                style = pango.STYLE_NORMAL
             else:
-                weight = pango.WEIGHT_BOLD
-            self.__model.append([key, self.type_as_string(fieldtype), str(value), fieldtype, fieldtype is not bool, weight])
+                style = pango.STYLE_ITALIC
+
+            self.__model.append([key, self.type_as_string(fieldtype), str(value), fieldtype, fieldtype is not bool, style, fieldtype is bool, bool(value)])
 
     def model(self):
         return self.__model
@@ -358,12 +367,14 @@ class Config(dict):
                             log('Error while calling observer: %s', repr(observer), sender=self)
                     for row in self.__model:
                         if row[0] == name:
-                            row[2] = str(fieldtype(value))
+                            value = fieldtype(value)
+                            row[2] = str(value)
+                            row[7] = bool(value)
                             if self[name] == default:
-                                weight = pango.WEIGHT_NORMAL
+                                style = pango.STYLE_NORMAL
                             else:
-                                weight = pango.WEIGHT_BOLD
-                            row[5] = weight
+                                style = pango.STYLE_ITALIC
+                            row[5] = style
                     self.schedule_save()
             except:
                 raise ValueError( '%s has to be of type %s' % ( name, fieldtype.__name__ ))
