@@ -35,12 +35,15 @@ import gpodder
 from gpodder import util
 from gpodder import opml
 from gpodder import config
+from gpodder import dumbshelve
+from gpodder.dbsqlite import db
 
 import os
 import os.path
 import glob
 import types
 import subprocess
+import sys
 
 from liblogger import log
 
@@ -60,9 +63,7 @@ class gPodderLib(object):
         util.make_directory( gpodder_dir)
 
         self.tempdir = gpodder_dir
-        self.feed_cache_file = os.path.join(gpodder_dir, 'feedcache.pickle.db')
         self.channel_settings_file = os.path.join(gpodder_dir, 'channelsettings.pickle.db')
-        self.episode_metainfo_file = os.path.join(gpodder_dir, 'episodemetainfo.pickle.db')
 
         self.channel_opml_file = os.path.join(gpodder_dir, 'channels.opml')
         self.channel_xml_file = os.path.join(gpodder_dir, 'channels.xml')
@@ -81,9 +82,127 @@ class gPodderLib(object):
         if self.config.videoplayer == 'unspecified':
             self.config.videoplayer = self.config.player	
 
-        self.__download_history = HistoryStore( os.path.join( gpodder_dir, 'download-history.txt'))
-        self.__playback_history = HistoryStore( os.path.join( gpodder_dir, 'playback-history.txt'))
-        self.__locked_history = HistoryStore( os.path.join( gpodder_dir, 'lock-history.txt'))
+        self.gpodder_dir = gpodder_dir
+        not db.setup({ 'database': os.path.join(gpodder_dir, 'database.sqlite') })
+
+    def migrate_to_sqlite(self, add_callback, status_callback, load_channels, get_localdb):
+        """
+        Migrates from the 0.11.3 data storage format
+        to the new SQLite-based storage format.
+
+        add_callback should accept one parameter:
+            + url (the url for a channel to be added)
+
+        status_callback should accept two parameters:
+            + percentage (a float, 0..100)
+            + message (current status message, a string)
+
+        load_channels should return the channel list
+
+        get_localdb should accept one parameter:
+            + channel (a channel object)
+            and should return a list of episodes
+        """
+        if os.path.exists(self.channel_opml_file):
+            channels = opml.Importer(gl.channel_opml_file).items
+        else:
+            channels = []
+
+        p = 0.0
+
+        # 0..40% -> import channels
+        if len(channels):
+            p_step = 40.0/len(channels)
+            for c in channels:
+                log('Importing %s', c['url'], sender=self)
+                status_callback(p, _('Adding podcast: %s') % c['title'])
+                add_callback(c['url'])
+                p += p_step
+        else:
+            p = 40.0
+
+        # 40..50% -> import localdb
+        channels = load_channels()
+        if len(channels):
+            p_step = 10.0/len(channels)
+            for channel in channels:
+                status_callback(p, _('Loading LocalDB for %s') % channel.title)
+                if os.path.exists(channel.index_file):
+                    episodes = get_localdb(channel)
+                else:
+                    episodes = []
+                if len(episodes):
+                    p_step_2 = p_step/len(episodes)
+                    for episode in episodes:
+                        ### status_callback(p, _('Adding episode: %s') % episode.title)
+                        # This, or all episodes will be marked as new after import.
+                        episode.is_played = True
+                        if (episode.file_exists()):
+                            episode.mark(state=db.STATE_DOWNLOADED)
+                        episode.save(bulk=True)
+                        p += p_step_2
+                    # flush the localdb updates for this channel
+                    status_callback(p, _('Writing changes to database'))
+                    db.commit()
+                else:
+                    p += p_step
+        else:
+            p += 10.0
+            
+        # 50..65% -> import download history
+        download_history = HistoryStore(os.path.join(self.gpodder_dir, 'download-history.txt'))
+        if len(download_history):
+            p_step = 15.0/len(download_history)
+            for url in download_history:
+                ### status_callback(p, _('Adding to history: %s') % url)
+                db.mark_episode(url, state=db.STATE_DELETED)
+                p += p_step
+        else:
+            p += 15.0
+
+        # 65..90% -> fix up all episode statuses
+        channels = load_channels()
+        if len(channels):
+            p_step = 25.0/len(channels)
+            for channel in channels:
+                status_callback(p, _('Migrating settings for %s') % channel.title)
+                ChannelSettings.migrate_settings(channel)
+                status_callback(p, _('Fixing episodes in %s') % channel.title)
+                all_episodes = channel.get_all_episodes()
+                if len(all_episodes):
+                    p_step_2 = p_step/len(all_episodes)
+                    for episode in all_episodes:
+                        ### status_callback(p, _('Checking episode: %s') % episode.title)
+                        if episode.state == db.STATE_DELETED and episode.file_exists():
+                            episode.mark(state=db.STATE_DOWNLOADED, is_played=False)
+                        # episode.fix_corrupted_state()
+                        p += p_step_2
+                else:
+                    p += p_step
+        else:
+            p += 25.0
+
+        # 90..95% -> import playback history
+        playback_history = HistoryStore(os.path.join(self.gpodder_dir, 'playback-history.txt'))
+        if len(playback_history):
+            p_step = 5.0/len(playback_history)
+            for url in playback_history:
+                ### status_callback(p, _('Playback history: %s') % url)
+                db.mark_episode(url, is_played=True)
+                p += p_step
+        else:
+            p += 5.0
+            
+        # 95..100% -> import locked history
+        locked_history = HistoryStore(os.path.join(self.gpodder_dir, 'lock-history.txt'))
+        if len(locked_history):
+            p_step = 5.0/len(locked_history)
+            for url in locked_history:
+                ### status_callback(p, _('Locked history: %s') % url)
+                db.mark_episode(url, is_locked=True)
+                p += p_step
+        else:
+            p += 5.0
 
     def migrate_channels_xml(self):
         """Migrate old (gPodder < 0.9.5) channels.xml to channels.opml
@@ -165,40 +284,19 @@ class gPodderLib(object):
                     os.rmdir( new_downloaddir)
 
                 shutil.move( self.config.download_dir, new_downloaddir)
-            except:
-                log( 'Error while moving %s to %s.', self.config.download_dir, new_downloaddir)
+            except NameError:
+                log( 'Fixing a bug in shutil. See http://bugs.python.org/issue2549')
+                errno = subprocess.call(["rm", "-rf", self.config.download_dir])
+                if errno <> 0:
+                    log( 'Error while deleting %s: rm returned error %i', self.config.download_dir, errno) 
+                    return
+            except Exception, exc:
+                log( 'Error while moving %s to %s: %s',self.config.download_dir, new_downloaddir, exc)
                 return
 
         self.config.download_dir = new_downloaddir
 
     downloaddir = property(fget=get_download_dir,fset=set_download_dir)
-
-    def history_mark_downloaded( self, url, add_item = True):
-        if add_item:
-            self.__download_history.add_item( url)
-        else:
-            self.__download_history.del_item( url)
-
-    def history_mark_played( self, url, add_item = True):
-        if add_item:
-            self.__playback_history.add_item( url)
-        else:
-            self.__playback_history.del_item( url)
-
-    def history_mark_locked( self, url, add_item = True):
-        if add_item:
-            self.__locked_history.add_item( url)
-        else:
-            self.__locked_history.del_item( url)
-
-    def history_is_downloaded( self, url):
-        return (url in self.__download_history)
-
-    def history_is_played( self, url):
-        return (url in self.__playback_history)
-
-    def history_is_locked( self, url):
-        return (url in self.__locked_history)
     
     def send_subscriptions(self):
         try:
@@ -210,7 +308,7 @@ class gPodderLib(object):
         return True
 
     def playback_episode( self, channel, episode):
-        self.history_mark_played( episode.url)
+        db.mark_episode(episode.url, is_played=True)
         filename = episode.local_filename()
 
         if gpodder.interface == gpodder.MAEMO and not self.config.maemo_allow_custom_player:
@@ -223,7 +321,7 @@ class gPodderLib(object):
             return (True, service)
 
         # Determine the file type and set the player accordingly.  
-        file_type = util.file_type_by_extension(util.file_extension_from_url(episode.url))
+        file_type = episode.file_type()
 
         if file_type == 'video':
             player = self.config.videoplayer
@@ -241,62 +339,8 @@ class gPodderLib(object):
             return ( False, command_line[0] )
         return ( True, command_line[0] )
 
-    def image_download_thread( self, url, callback_pixbuf = None, callback_status = None, callback_finished = None, cover_file = None):
-        if callback_status is not None:
-            util.idle_add(callback_status, _('Downloading podcast cover...'))
-        pixbuf = gtk.gdk.PixbufLoader()
-        
-        if cover_file is None:
-            log( 'Downloading %s', url)
-            pixbuf.write( urllib.urlopen(url).read())
-        
-        if cover_file is not None and not os.path.exists(cover_file):
-            log( 'Downloading cover to %s', cover_file)
-            cachefile = open( cover_file, "w")
-            cachefile.write( urllib.urlopen(url).read())
-            cachefile.close()
-        
-        if cover_file is not None:
-            log( 'Reading cover from %s', cover_file)
-            try:
-                pixbuf.write( open( cover_file, "r").read())
-            except:
-                # Probably a data error, delete temp file
-                log('Data error while reading pixbuf. Deleting %s', cover_file, sender=self)
-                util.delete_file(cover_file)
-        
-        try:
-            pixbuf.close()
-        except:
-            # data error, delete temp file
-            util.delete_file( cover_file)
-        
-        MAX_SIZE = 400
-        if callback_pixbuf is not None:
-            pb = pixbuf.get_pixbuf()
-            if pb:
-                if pb.get_width() > MAX_SIZE:
-                    factor = MAX_SIZE*1.0/pb.get_width()
-                    pb = pb.scale_simple( int(pb.get_width()*factor), int(pb.get_height()*factor), gtk.gdk.INTERP_BILINEAR)
-                if pb.get_height() > MAX_SIZE:
-                    factor = MAX_SIZE*1.0/pb.get_height()
-                    pb = pb.scale_simple( int(pb.get_width()*factor), int(pb.get_height()*factor), gtk.gdk.INTERP_BILINEAR)
-                util.idle_add(callback_pixbuf, pb)
-        if callback_status is not None:
-            util.idle_add(callback_status, '')
-        if callback_finished is not None:
-            util.idle_add(callback_finished)
-
-    def get_image_from_url( self, url, callback_pixbuf = None, callback_status = None, callback_finished = None, cover_file = None):
-        if not url and not os.path.exists( cover_file):
-            return
-
-        args = ( url, callback_pixbuf, callback_status, callback_finished, cover_file )
-        thread = threading.Thread( target = self.image_download_thread, args = args)
-        thread.start()
-
     def invoke_torrent( self, url, torrent_filename, target_filename):
-        self.history_mark_played( url)
+        db.mark_episode(url, is_played=True)
 
         if self.config.use_gnome_bittorrent:
             if util.find_command('gnome-btdownload') is None:
@@ -318,8 +362,43 @@ class gPodderLib(object):
 
         return False
 
+    def ext_command_thread(self, notification, command_line):
+        """
+        This is the function that will be called in a separate
+        thread that will call an external command (specified by
+        command_line). In case of problem (i.e. the command has
+        not been found or there has been another error), we will
+        call the notification function with two arguments - the
+        first being the error message and the second being the
+        title to be used for the error message.
+        """
+ 
+        log("(ExtCommandThread) Excuting command Line [%s]", command_line)
+    
+        p = subprocess.Popen(command_line, shell=True, stdout=sys.stdout, stderr=sys.stderr)
+        result = p.wait()
+ 
+        if result == 127:
+            title = _('User command not found')
+            message = _('The user command [%s] was not found.\nPlease check your user command settings in the preferences dialog.' % command_line)
+            notification(message, title)
+        elif result == 126:
+            title = _('User command permission denied')
+            message = _('Permission denied when trying to execute user command [%s].\nPlease check that you have permissions to execute this command.' % command_line)
+            notification(message, title)
+        elif result > 0 :
+            title = _('User command returned an error')
+            message = _('The user command [%s] returned an error code of [%d]' % (command_line,result))
+            notification(message, title)
+
+        log("(ExtCommandThread) Finished command line [%s] result [%d]",command_line,result)
+
 
 class HistoryStore( types.ListType):
+    """
+    DEPRECATED - Only used for migration to SQLite
+    """
+
     def __init__( self, filename):
         self.filename = filename
         try:
@@ -374,6 +453,40 @@ class HistoryStore( types.ListType):
         return affected
 
 
+class ChannelSettings(object):
+    """
+    DEPRECATED - Only used for migration to SQLite
+    """
+    SETTINGS_TO_MIGRATE = ('sync_to_devices', 'override_title', 'username', 'password')
+    storage = None
+
+    @classmethod
+    def migrate_settings(cls, channel):
+        url = channel.url
+        settings = {}
+
+        if cls.storage is None:
+            if os.path.exists(gl.channel_settings_file):
+                cls.storage = dumbshelve.open_shelve(gl.channel_settings_file)
+
+        # We might have failed to open the shelve if we didn't have a settings
+        # file in the first place (e.g., the user just deleted the database and
+        # reimports everything from channels.opml).
+        if cls.storage is not None:
+            if isinstance(url, unicode):
+                url = url.encode('utf-8')
+            if cls.storage.has_key(url):
+                settings = cls.storage[url]
+
+            if settings:
+                log('Migrating settings for %s', url)
+            for key in cls.SETTINGS_TO_MIGRATE:
+                if settings.has_key(key):
+                    log('Migrating key %s', key)
+                    setattr(channel, key, settings[key])
+
+
 # Global, singleton gPodderLib object
 gl = gPodderLib()
 
+
