@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # gPodder - A media aggregator and podcast client
-# Copyright (C) 2005-2007 Thomas Perl <thp at perli.net>
+# Copyright (c) 2005-2008 Thomas Perl and the gPodder Team
 #
 # gPodder is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #
 
 import traceback
+import time
 
 write_to_stdout = False
 
@@ -32,12 +33,20 @@ def enable_verbose():
     global write_to_stdout
     write_to_stdout = True
 
+first_time = time.time()
+last_times = []
 
 def log( message, *args, **kwargs):
+    global first_time
+    global last_times
     if 'sender' in kwargs:
         message = '(%s) %s' % ( kwargs['sender'].__class__.__name__, message )
+    if 'bench_start' in kwargs:
+        last_times.append(time.time())
+    if 'bench_end' in kwargs and len(last_times) > 0:
+        message += (' (benchmark: %.4f seconds)' % (time.time()-(last_times.pop())))
     if write_to_stdout:
-        print message % args
+        print (('[%8.3f] ' % (time.time()-first_time)) + message) % args
         if kwargs.get( 'traceback', False):
             error = traceback.format_exc()
             if error.strip() != 'None':
