@@ -33,6 +33,9 @@ import gtk
 import gobject
 
 import threading
+import urllib2
+import os
+import os.path
 
 
 class ObservableService(object):
@@ -67,6 +70,130 @@ class ObservableService(object):
             log('Signal "%s" is not available for notification.', signal_name, sender=self)
 
 
+class CoverDownloader(ObservableService):
+    """
+    This class manages downloading cover art and notification
+    of other parts of the system. Downloading cover art can
+    happen either synchronously via get_cover() or in
+    asynchronous mode via request_cover(). When in async mode,
+    the cover downloader will send the cover via the
+    'cover-available' message (via the ObservableService).
+    """
+
+    # Maximum width/height of the cover in pixels
+    MAX_SIZE = 400
+
+    def __init__(self):
+        signal_names = ['cover-available', 'cover-removed']
+        ObservableService.__init__(self, signal_names)
+
+    def request_cover(self, channel, custom_url=None):
+        """
+        Sends an asynchronous request to download a
+        cover for the specific channel.
+
+        After the cover has been downloaded, the
+        "cover-available" signal will be sent with
+        the channel url and new cover as pixbuf.
+
+        If you specify a custom_url, the cover will
+        be downloaded from the specified URL and not
+        taken from the channel metadata.
+        """
+        args = [channel, custom_url, True]
+        threading.Thread(target=self.__get_cover, args=args).start()
+
+    def get_cover(self, channel, custom_url=None, avoid_downloading=False):
+        """
+        Sends a synchronous request to download a
+        cover for the specified channel.
+
+        The cover will be returned to the caller.
+
+        The custom_url has the same semantics as
+        in request_cover().
+
+        The optional parameter "avoid_downloading",
+        when true, will make sure we return only
+        already-downloaded covers and return None
+        when we have no cover on the local disk.
+        """
+        (url, pixbuf) = self.__get_cover(channel, custom_url, False, avoid_downloading)
+        return pixbuf
+
+    def remove_cover(self, channel):
+        """
+        Removes the current cover for the channel
+        so that a new one is downloaded the next
+        time we request the channel cover.
+        """
+        util.delete_file(channel.cover_file)
+        self.notify('cover-removed', channel.url)
+
+    def replace_cover(self, channel, custom_url=None):
+        """
+        This is a convenience function that deletes
+        the current cover file and requests a new
+        cover from the URL specified.
+        """
+        self.remove_cover(channel)
+        self.request_cover(channel, custom_url)
+
+    def __get_cover(self, channel, url, async=False, avoid_downloading=False):
+        if not async and avoid_downloading and not os.path.exists(channel.cover_file):
+            return (channel.url, None)
+
+        loader = gtk.gdk.PixbufLoader()
+        pixbuf = None
+
+        if not os.path.exists(channel.cover_file):
+            if url is None:
+                url = channel.image
+
+            if url is not None:
+                image_data = None
+                try:
+                    log('Trying to download: %s', url, sender=self)
+
+                    image_data = urllib2.urlopen(url).read()
+                except:
+                    log('Cannot get image from %s', url, sender=self)
+             
+                if image_data is not None:
+                    log('Saving image data to %s', channel.cover_file, sender=self)
+                    fp = open(channel.cover_file, 'wb')
+                    fp.write(image_data)
+                    fp.close()
+
+        if os.path.exists(channel.cover_file):
+            loader.write(open(channel.cover_file, 'rb').read())
+            try:
+                loader.close()
+                pixbuf = loader.get_pixbuf()
+            except:
+                log('Data error while loading %s', channel.cover_file, sender=self)
+        else:
+            try:
+                loader.close()
+            except:
+                pass
+
+        if pixbuf is not None:
+            new_pixbuf = util.resize_pixbuf_keep_ratio(pixbuf, self.MAX_SIZE, self.MAX_SIZE)
+            if new_pixbuf is not None:
+                # Save the resized cover so we do not have to
+                # resize it next time we load it
+                new_pixbuf.save(channel.cover_file, 'png')
+                pixbuf = new_pixbuf
+
+        if async:
+            self.notify('cover-available', channel.url, pixbuf)
+        else:
+            return (channel.url, pixbuf)
+
+cover_downloader = CoverDownloader()
+
+
 class DownloadStatusManager(ObservableService):
     COLUMN_NAMES = { 0: 'episode', 1: 'speed', 2: 'progress', 3: 'url' }
     COLUMN_TYPES = ( gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_FLOAT, gobject.TYPE_STRING )
