@@ -30,10 +30,13 @@ from gpodder import libtagupdate
 
 from gpodder.liblogger import log
 from gpodder.libgpodder import gl
+from gpodder.dbsqlite import db
 
+gpod_available = True
 try:
     import gpod
 except:
+    gpod_available = False
     log('(gpodder.sync) Could not find gpod')
 
 try:
@@ -46,6 +49,10 @@ try:
 except:
     log( '(gpodder.sync) Could not find eyeD3')
 
+try:
+    import Image
+except:
+    log('(gpodder.sync) Could not find Python Imaging Library (PIL)')
 
 import os
 import os.path
@@ -129,7 +136,8 @@ class Device(services.ObservableService):
     def __init__(self):
         self.cancelled = False
         self.allowed_types = ['audio', 'video']
-
+        self.errors = []
+        self.tracks_list = []
         signals = ['progress', 'sub-progress', 'status', 'done']
         services.ObservableService.__init__(self, signals)
 
@@ -153,10 +161,10 @@ class Device(services.ObservableService):
 
             self.notify('progress', id+1, len(tracklist))
 
-            if not track.is_downloaded():
+            if not track.was_downloaded(and_exists=True):
                 continue
 
-            if track.is_played() and gl.config.only_sync_not_played and not force_played:
+            if track.is_played and gl.config.only_sync_not_played and not force_played:
                 continue
 
             if track.file_type() not in self.allowed_types:
@@ -166,7 +174,7 @@ class Device(services.ObservableService):
 
             if gl.config.on_sync_mark_played:
                 log('Marking as played on transfer: %s', track.url, sender=self)
-                gl.history_mark_played(track.url)
+                db.mark_episode(track.url, is_played=True)
 
             if added and gl.config.on_sync_delete:
                 log('Removing episode after transfer: %s', track.url, sender=self)
@@ -190,6 +198,17 @@ class Device(services.ObservableService):
     def remove_track(self, track):
         pass
 
+    def get_free_space(self):
+        pass
+
+    def episode_on_device(self, episode):
+        return self._track_on_device(episode)
+
+    def _track_on_device( self, track_name ):
+        for t in self.tracks_list:
+            if track_name == t.title:
+                return t
+        return False
 
 class iPodDevice(Device):
     def __init__(self):
@@ -199,6 +218,12 @@ class iPodDevice(Device):
 
         self.itdb = None
         self.podcast_playlist = None
+        
+
+    def get_free_space(self):
+        # Reserve 10 MiB for iTunesDB writing (to be on the safe side)
+        RESERVED_FOR_ITDB = 1024*1024*10
+        return util.get_free_disk_space(self.mountpoint) - RESERVED_FOR_ITDB
 
     def open(self):
         if not os.path.isdir(self.mountpoint):
@@ -214,6 +239,10 @@ class iPodDevice(Device):
 
         if self.podcasts_playlist:
             self.notify('status', _('iPod opened'))
+
+            # build the initial tracks_list
+            self.tracks_list = self.get_all_tracks()
+
             return True
         else:
             return False
@@ -259,7 +288,7 @@ class iPodDevice(Device):
 
             t = SyncTrack(track.title, length, modified, libgpodtrack=track, playcount=track.playcount, released=released, podcast=track.artist)
             tracks.append(t)
-        return tracks
+        return tracks        
 
     def remove_track(self, track):
         self.notify('status', _('Removing %s') % track.title)
@@ -279,7 +308,7 @@ class iPodDevice(Device):
         for track in gpod.sw_get_playlist_tracks(self.podcasts_playlist):
             if episode.url == track.podcasturl:
                 if track.playcount > 0:
-                    gl.history_mark_played(track.podcasturl)
+                    db.mark_episode(track.podcasturl, is_played=True)
                 # Mark as played on iPod if played locally (and set podcast flags)
                 self.set_podcast_flags(track)
                 return True
@@ -287,13 +316,8 @@ class iPodDevice(Device):
         original_filename = str(episode.local_filename())
         local_filename = original_filename
 
-        # Reserve 10 MiB for iTunesDB writing (to be on the safe side)
-        RESERVED_FOR_ITDB = 1024*1024*10
-        space_for_track = util.get_free_disk_space(self.mountpoint) - RESERVED_FOR_ITDB
-        needed = util.calculate_size(local_filename)
-
-        if needed > space_for_track:
-            log('Not enough space on %s: %s available, but need at least %s', self.mountpoint, util.format_filesize(space_for_track), util.format_filesize(needed), sender = self)
+        if util.calculate_size(original_filename) > self.get_free_space():
+            log('Not enough space on %s, sync aborted...', self.mountpoint, sender = self)
             self.errors.append( _('Error copying %s: Not enough free disk space on %s') % (episode.title, self.mountpoint))
             self.cancelled = True
             return False
@@ -320,18 +344,17 @@ class iPodDevice(Device):
 
         track = gpod.itdb_track_new()
         
-        # Add release time to track if pubDate is parseable
-        ipod_date = email.Utils.parsedate(episode.pubDate)
-        if ipod_date is not None:
+        # Add release time to track if pubDate has a valid value
+        if episode.pubDate > 0:
             try:
                 # libgpod>= 0.5.x uses a new timestamp format
-                track.time_released = gpod.itdb_time_host_to_mac(int(time.mktime(ipod_date)))
+                track.time_released = gpod.itdb_time_host_to_mac(int(episode.pubDate))
             except:
                 # old (pre-0.5.x) libgpod versions expect mactime, so
                 # we're going to manually build a good mactime timestamp here :)
                 #
                 # + 2082844800 for unixtime => mactime (1970 => 1904)
-                track.time_released = int(time.mktime(ipod_date) + 2082844800)
+                track.time_released = int(episode.pubDate + 2082844800)
         
         track.title = str(episode.title)
         track.album = str(episode.channel.title)
@@ -367,7 +390,8 @@ class iPodDevice(Device):
     def set_podcast_flags(self, track):
         try:
             # Set blue bullet for unplayed tracks on 5G iPods
-            if gl.history_is_played(track.podcasturl):
+            episode = db.load_episode(track.podcasturl)
+            if episode['is_played']:
                 track.mark_unplayed = 0x01
                 if track.playcount == 0:
                     track.playcount = 1
@@ -423,32 +447,23 @@ class MP3PlayerDevice(Device):
     # This is the maximum length of a file name that is
     # created on the MP3 player, because FAT32 has a
     # 255-character limit for the whole path
-    MAX_FILENAME_LENGTH = 100
+    MAX_FILENAME_LENGTH = gl.config.mp3_player_max_filename_length
 
     def __init__(self):
         Device.__init__(self)
-
-        # Try to detect OS encoding (by Leonid Ponomarev)
-        if 'LANG' in os.environ and '.' in os.environ['LANG']:
-            lang = os.environ['LANG']
-            (language, encoding) = lang.rsplit('.', 1)
-            log('Detected encoding: %s', encoding, sender=self)
-            self.enc = encoding
-        else:
-            # Using iso-8859-15 here as (hopefully) sane default
-            # see http://en.wikipedia.org/wiki/ISO/IEC_8859-1
-            log('Using ISO-8859-15 as encoding. If this', sender=self)
-            log('is incorrect, please set your $LANG variable.', sender=self)
-            self.enc = 'iso-8859-15'
-
         self.destination = gl.config.mp3_player_folder
         self.buffer_size = 1024*1024 # 1 MiB
         self.scrobbler_log = []
 
+    def get_free_space(self):
+        return util.get_free_disk_space(self.destination)
+
     def open(self):
         self.notify('status', _('Opening MP3 player'))
         if util.directory_is_writable(self.destination):
             self.notify('status', _('MP3 player opened'))
+            # build the initial tracks_list
+            self.tracks_list = self.get_all_tracks()
             if gl.config.mp3_player_use_scrobbler_log:
                 mp3_player_mount_point = util.find_mount_point(self.destination)
                 # If a moint point cannot be found look inside self.destination for scrobbler_log_filenames
@@ -468,31 +483,17 @@ class MP3PlayerDevice(Device):
         if gl.config.fssync_channel_subfolders:
             # Add channel title as subfolder
             folder = episode.channel.title
-            # Don't allow extremely long folder names (filesystem limitations)
-            if len(folder) > self.MAX_FILENAME_LENGTH:
-                log('Limiting folder "%s" to %d characters.', folder, self.MAX_FILENAME_LENGTH, sender=self)
-                folder = folder[:self.MAX_FILENAME_LENGTH]
-            folder = re.sub('[/|?*<>:+\[\]\"\\\]', '_', folder.encode(self.enc, 'ignore'))
+            # Clean up the folder name for use on limited devices
+            folder = util.sanitize_filename(folder, self.MAX_FILENAME_LENGTH)
             folder = os.path.join(self.destination, folder)
         else:
             folder = self.destination
 
         from_file = episode.local_filename()
-
-        filename_base = episode.sync_filename()
-
-        # don't allow extremely long file names (filesystem limitations)
-        if len(filename_base) > self.MAX_FILENAME_LENGTH:
-            log('Limiting filename "%s" to %d characters.', filename_base, self.MAX_FILENAME_LENGTH, sender=self)
-            filename_base = filename_base[:self.MAX_FILENAME_LENGTH]
+        filename_base = util.sanitize_filename(episode.sync_filename(), self.MAX_FILENAME_LENGTH)
 
         to_file = filename_base + os.path.splitext(from_file)[1].lower()
 
-        # Encode the file name to our system's
-        # encoding and remove the characters that are invalid
-        # for FAT-based drives (replace with the empty string)
-        to_file = re.sub('[/|?*<>:+\[\]\"\\\]', '_', to_file.encode(self.enc, 'ignore'))
-
         # dirty workaround: on bad (empty) episode titles,
         # we simply use the from_file basename
         # (please, podcast authors, FIX YOUR RSS FEEDS!)
@@ -508,13 +509,17 @@ class MP3PlayerDevice(Device):
                 log('Cannot create folder on MP3 player: %s', folder, sender=self)
                 return False
 
-        if (gl.config.mp3_player_use_scrobbler_log and not episode.is_played()
+        if (gl.config.mp3_player_use_scrobbler_log and not episode.is_played
                 and [episode.channel.title, episode.title] in self.scrobbler_log):
             log('Marking "%s" from "%s" as played', episode.title, episode.channel.title, sender=self)
-            gl.history_mark_played(episode.url)
+            db.mark_episode(episode.url, is_played=True)
+
+        if gl.config.rockbox_copy_coverart and not os.path.exists(os.path.join(folder, 'cover.bmp')):
+            log('Creating Rockbox album art for "%s"', episode.channel.title, sender=self)
+            self.copy_rockbox_cover_art(folder, from_file)
 
         if not os.path.exists(to_file):
-            log('Copying %s => %s', os.path.basename(from_file), to_file.decode(self.enc), sender=self)
+            log('Copying %s => %s', os.path.basename(from_file), to_file.decode(util.encoding), sender=self)
             return self.copy_file_progress(from_file, to_file)
 
         return True
@@ -588,6 +593,10 @@ class MP3PlayerDevice(Device):
             tracks.append(t)
         return tracks
 
+    def episode_on_device(self, episode):
+        e = util.sanitize_filename(episode.sync_filename(), gl.config.mp3_player_max_filename_length)
+        return self._track_on_device(e)
+
     def remove_track(self, track):
         self.notify('status', _('Removing %s') % track.title)
         util.delete_file(track.filename)
@@ -614,6 +623,35 @@ class MP3PlayerDevice(Device):
         # No scrobbler log on that device
         return None
 
+    def copy_rockbox_cover_art(self, destination, local_filename):
+        """
+        Try to copy the channel cover to "cover.bmp" in the podcast
+        folder on the MP3 player. This makes Rockbox (rockbox.org) display
+        the cover art in its interface.
+
+        You need the Python Imaging Library (PIL) installed to be able to
+        convert the cover file to a Bitmap file, which Rockbox needs.
+        """
+        try:
+            cover_loc = os.path.join(os.path.dirname(local_filename), 'cover')
+            cover_dst = os.path.join(destination, 'cover.bmp')
+            if os.path.isfile(cover_loc):
+                size = (gl.config.rockbox_coverart_size, gl.config.rockbox_coverart_size)
+                try:
+                    cover = Image.open(cover_loc)
+                    cover.thumbnail(size)
+                    cover.save(cover_dst, 'BMP')
+                except IOError:
+                    log('Cannot create %s (PIL?)', cover_dst, traceback=True, sender=self)
+                return True
+            else:
+                log('No cover available to set as Rockbox cover', sender=self)
+                return True
+        except:
+            log('Error getting cover using channel cover', sender=self)
+        return False
+
+
     def load_audioscrobbler_log(self, log_file):
         """ Retrive track title and artist info for all the entries
             in an audioscrobbler portable player format logfile
