@@ -54,6 +54,7 @@ import urllib
 import urllib2
 import httplib
 import webbrowser
+import mimetypes
 
 import feedparser
 
@@ -61,12 +62,28 @@ import StringIO
 import xml.dom.minidom
 
 
+# Try to detect OS encoding (by Leonid Ponomarev)
+encoding = 'iso-8859-15'
+if 'LANG' in os.environ and '.' in os.environ['LANG']:
+    lang = os.environ['LANG']
+    (language, encoding) = lang.rsplit('.', 1)
+    log('Detected encoding: %s', encoding)
+    enc = encoding
+else:
+    # Using iso-8859-15 here as (hopefully) sane default
+    # see http://en.wikipedia.org/wiki/ISO/IEC_8859-1
+    log('Using ISO-8859-15 as encoding. If this')
+    log('is incorrect, please set your $LANG variable.')
+
+
 if gpodder.interface == gpodder.GUI:
     ICON_UNPLAYED = gtk.STOCK_YES
     ICON_LOCKED = 'emblem-nowrite'
+    ICON_MISSING = gtk.STOCK_STOP
 elif gpodder.interface == gpodder.MAEMO:
     ICON_UNPLAYED = 'qgn_list_gene_favor'
     ICON_LOCKED = 'qgn_indi_KeypadLk_lock'
+    ICON_MISSING = gtk.STOCK_STOP # FIXME!
 
 def make_directory( path):
     """
@@ -242,7 +259,6 @@ def get_free_disk_space(path):
     function returns zero.
     """
 
-    path = os.path.dirname(path)
     if not os.path.exists(path):
         return 0
 
@@ -261,18 +277,27 @@ def format_date(timestamp):
     Returns None if there has been an error converting the
     timestamp to a string representation.
     """
+    if timestamp is None:
+        return None
+
     seconds_in_a_day = 60*60*24
+
+    today = time.localtime()[:3]
+    yesterday = time.localtime(time.time() - seconds_in_a_day)[:3]
+    timestamp_date = time.localtime(timestamp)[:3]
+    
+    if timestamp_date == today:
+       return _('Today')
+    elif timestamp_date == yesterday:
+       return _('Yesterday')
+   
     try:
-        diff = int((time.time()+1)/seconds_in_a_day) - int(timestamp/seconds_in_a_day)
+        diff = int( (time.time() - timestamp)/seconds_in_a_day )
     except:
         log('Warning: Cannot convert "%s" to date.', timestamp, traceback=True)
         return None
-    
-    if diff == 0:
-       return _('Today')
-    elif diff == 1:
-       return _('Yesterday')
-    elif diff < 7:
+
+    if diff < 7:
         # Weekday name
         return str(datetime.datetime.fromtimestamp(timestamp).strftime('%A'))
     else:
@@ -350,15 +375,27 @@ def remove_html_tags(html):
     re_strip_tags = re.compile('<[^>]*>')
     re_unicode_entities = re.compile('&#(\d{2,4});')
     re_html_entities = re.compile('&(.{2,8});')
+    re_newline_tags = re.compile('(<br[^>]*>|<[/]?ul[^>]*>|</li>)', re.I)
+    re_listing_tags = re.compile('<li[^>]*>', re.I)
+
+    result = html
+    
+    # Convert common HTML elements to their text equivalent
+    result = re_newline_tags.sub('\n', result)
+    result = re_listing_tags.sub('\n * ', result)
+    result = re.sub('<[Pp]>', '\n\n', result)
 
     # Remove all HTML/XML tags from the string
-    result = re_strip_tags.sub('', html)
+    result = re_strip_tags.sub('', result)
 
     # Convert numeric XML entities to their unicode character
     result = re_unicode_entities.sub(lambda x: unichr(int(x.group(1))), result)
 
     # Convert named HTML entities to their unicode character
     result = re_html_entities.sub(lambda x: unicode(entitydefs.get(x.group(1),''), 'iso-8859-1'), result)
+    
+    # Convert more than two newlines to two newlines
+    result = re.sub('([\r\n]{2})([\r\n])+', '\\1', result)
 
     return result.strip()
 
@@ -387,12 +424,17 @@ def torrent_filename( filename):
     except:
         return None
 
-
-def file_extension_from_url( url):
+def extension_from_mimetype(mimetype):
     """
-    Extracts the (lowercase) file name extension (with dot)
+    Simply guesses what the file extension should be from the mimetype
+    """
+    return mimetypes.guess_extension(mimetype) or ''
+
+def filename_from_url(url):
+    """
+    Extracts the filename and (lowercase) extension (with dot)
     from a URL, e.g. http://server.com/file.MP3?download=yes
-    will result in the string ".mp3" being returned.
+    will result in the string ("file", ".mp3") being returned.
 
     This function will also try to best-guess the "real" 
     extension for a media file (audio, video, torrent) by 
@@ -400,27 +442,29 @@ def file_extension_from_url( url):
     into the query string to find better matches, if the 
     original extension does not resolve to a known type.
 
-    http://my.net/redirect.php?my.net/file.ogg => ".ogg"
-    http://server/get.jsp?file=/episode0815.MOV => ".mov"
+    http://my.net/redirect.php?my.net/file.ogg => ("file", ".ogg")
+    http://server/get.jsp?file=/episode0815.MOV => ("episode0815", ".mov")
+    http://s/redirect.mp4?http://serv2/test.mp4 => ("test", ".mp4")
     """
     (scheme, netloc, path, para, query, fragid) = urlparse.urlparse(url)
-    filename = os.path.basename( urllib.unquote(path))
-    (filename, extension) = os.path.splitext(filename)
+    (filename, extension) = os.path.splitext(os.path.basename( urllib.unquote(path)))
 
-    if file_type_by_extension(extension) is not None:
+    if file_type_by_extension(extension) is not None and not \
+        query.startswith(scheme+'://'):
         # We have found a valid extension (audio, video, torrent)
-        return extension.lower()
-    
+        # and the query string doesn't look like a URL
+        return ( filename, extension.lower() )
+
     # If the query string looks like a possible URL, try that first
     if len(query.strip()) > 0 and query.find('/') != -1:
         query_url = '://'.join((scheme, urllib.unquote(query)))
-        query_extension = file_extension_from_url(query_url)
+        (query_filename, query_extension) = filename_from_url(query_url)
 
         if file_type_by_extension(query_extension) is not None:
-            return query_extension
+            return os.path.splitext(os.path.basename(query_url))
 
-    # No exact match found, simply return the original extension
-    return extension.lower()
+    # No exact match found, simply return the original filename & extension
+    return ( filename, extension.lower() )
 
 
 def file_type_by_extension( extension):
@@ -451,7 +495,7 @@ def file_type_by_extension( extension):
     return None
 
 
-def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=None, icon_size=32):
+def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, add_missing=False, icon_cache=None, icon_size=32):
     """
     Loads an icon from the current icon theme at the specified
     size, suitable for display in a gtk.TreeView.
@@ -466,7 +510,7 @@ def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=Non
     the cache is supplied again and the icon is found in 
     the cache.
     """
-    global ICON_UNPLAYED, ICON_LOCKED
+    global ICON_UNPLAYED, ICON_LOCKED, ICON_MISSING
 
     if icon_cache is not None and (icon_name,add_bullet,add_padlock,icon_size) in icon_cache:
         return icon_cache[(icon_name,add_bullet,add_padlock,icon_size)]
@@ -479,9 +523,20 @@ def get_tree_icon(icon_name, add_bullet=False, add_padlock=False, icon_cache=Non
         log( '(get_tree_icon) Warning: Cannot load icon with name "%s", will use  default icon.', icon_name)
         icon = icon_theme.load_icon(gtk.STOCK_DIALOG_QUESTION, icon_size, 0)
 
-    if icon and (add_bullet or add_padlock):
+    if icon and (add_bullet or add_padlock or add_missing):
         # We'll modify the icon, so use .copy()
-        if add_bullet:
+        if add_missing:
+            log('lalala')
+            try:
+                icon = icon.copy()
+                emblem = icon_theme.load_icon(ICON_MISSING, int(float(icon_size)*1.2/3.0), 0)
+                (width, height) = (emblem.get_width(), emblem.get_height())
+                xpos = icon.get_width() - width
+                ypos = icon.get_height() - height
+                emblem.composite(icon, xpos, ypos, width, height, xpos, ypos, 1, 1, gtk.gdk.INTERP_BILINEAR, 255)
+            except:
+                log('(get_tree_icon) Error adding emblem to icon "%s".', icon_name)
+        elif add_bullet:
             try:
                 icon = icon.copy()
                 emblem = icon_theme.load_icon(ICON_UNPLAYED, int(float(icon_size)*1.2/3.0), 0)
@@ -514,29 +569,6 @@ def get_first_line( s):
     return s.strip().split('\n')[0].strip()
 
 
-def updated_parsed_to_rfc2822( updated_parsed):
-    """
-    Converts a 9-tuple from feedparser's updated_parsed 
-    field to a C-locale string suitable for further use.
-
-    If the updated_parsed field is None or not a 9-tuple,
-    this function returns None.
-    """
-    if updated_parsed is None or len(updated_parsed) != 9:
-        return None
-
-    old_locale = locale.getlocale( locale.LC_TIME)
-    locale.setlocale( locale.LC_TIME, 'C')
-    result = time.strftime( '%a, %d %b %Y %H:%M:%S GMT', updated_parsed)
-    if old_locale != (None, None):
-        try:
-            locale.setlocale( locale.LC_TIME, old_locale)
-        except:
-            log('Cannot revert locale to (%s, %s)', *old_locale)
-            pass
-    return result
-
-
 def object_string_formatter( s, **kwargs):
     """
     Makes attributes of object passed in as keyword 
@@ -837,7 +869,7 @@ def get_episode_info_from_url(url, proxy=None):
     could be parsed from the URL. This currently contains:
     
       "length": The size of the file in bytes
-      "pubdate": A formatted representation of the pubDate
+      "pubdate": The unix timestamp for the pubdate
 
     If the "proxy" parameter is used, it has to be the URL 
     of the HTTP proxy server to use, e.g. http://proxy:8080/
@@ -873,7 +905,7 @@ def get_episode_info_from_url(url, proxy=None):
     if 'last-modified' in r.msg:
         try:
             parsed_date = feedparser._parse_date(r.msg['last-modified'])
-            pubdate = updated_parsed_to_rfc2822(parsed_date)
+            pubdate = time.mktime(parsed_date)
             result['pubdate'] = pubdate
         except:
             log('Error converting last-modified header.')
@@ -891,7 +923,7 @@ def gui_open(filename):
         # FIXME: Win32-specific "open" code needed here
         # as fallback when xdg-open not available
     except:
-        log('Cannot open file/folder: "%s"', folder, sender=self, traceback=True)
+        log('Cannot open file/folder: "%s"', filename, sender=self, traceback=True)
 
 
 def open_website(url):
@@ -903,26 +935,19 @@ def open_website(url):
     threading.Thread(target=webbrowser.open, args=(url,)).start()
 
 
-def sanitize_filename(filename):
+def sanitize_filename(filename, max_length=0):
     """
     Generate a sanitized version of a filename that can
-    be written on disk (i.e. remove/replace invalid 
-    characters and encode in the native language)
+    be written on disk (i.e. remove/replace invalid
+    characters and encode in the native language) and
+    trim filename if greater than max_length (0 = no limit).
     """
-    # Try to detect OS encoding (by Leonid Ponomarev)
-    if 'LANG' in os.environ and '.' in os.environ['LANG']:
-        lang = os.environ['LANG']
-        (language, encoding) = lang.rsplit('.', 1)
-        log('Detected encoding: %s', encoding)
-        enc = encoding
-    else:
-        # Using iso-8859-15 here as (hopefully) sane default
-        # see http://en.wikipedia.org/wiki/ISO/IEC_8859-1
-        log('Using ISO-8859-15 as encoding. If this')
-        log('is incorrect, please set your $LANG variable.')
-        enc = 'iso-8859-15'
+    if max_length > 0 and len(filename) > max_length:
+        log('Limiting file/folder name "%s" to %d characters.', filename, max_length)
+        filename = filename[:max_length]
 
-    return re.sub('[/|?*<>:+\[\]\"\\\]', '_', filename.strip().encode(enc, 'ignore'))
+    global encoding
+    return re.sub('[/|?*<>:+\[\]\"\\\]', '_', filename.strip().encode(encoding, 'ignore'))
 
 
 def find_mount_point(directory):
@@ -941,3 +966,44 @@ def find_mount_point(directory):
 
     return '/'
 
+
+def resize_pixbuf_keep_ratio(pixbuf, max_width, max_height, key=None, cache=None):
+    """
+    Resizes a GTK Pixbuf but keeps its aspect ratio.
+    
+    Returns None if the pixbuf does not need to be
+    resized or the newly resized pixbuf if it does.
+
+    The optional parameter "key" is used to identify
+    the image in the "cache", which is a dict-object
+    that holds already-resized pixbufs to access.
+    """
+    changed = False
+
+    if cache is not None:
+        if (key, max_width, max_height) in cache:
+            return cache[(key, max_width, max_height)]
+
+    # Resize if too wide
+    if pixbuf.get_width() > max_width:
+        f = float(max_width)/pixbuf.get_width()
+        (width, height) = (int(pixbuf.get_width()*f), int(pixbuf.get_height()*f))
+        pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
+        changed = True
+
+    # Resize if too high
+    if pixbuf.get_height() > max_height:
+        f = float(max_height)/pixbuf.get_height()
+        (width, height) = (int(pixbuf.get_width()*f), int(pixbuf.get_height()*f))
+        pixbuf = pixbuf.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
+        changed = True
+
+    if changed:
+        result = pixbuf
+        if cache is not None:
+            cache[(key, max_width, max_height)] = result
+    else:
+        result = None
+
+    return result
+
